# -*- coding: utf-8 -*-
"""
``pybill.lib.entities`` package defines the PyBill entities used all over
PyBill.

These entities represent the bills, the claim forms, the debits, the
downpayments, the pro-formas and all their inner objects.
"""
__docformat__ = "restructuredtext en"
